@@ -6,8 +6,6 @@ mod compile_options;
 mod lane;
 mod module;
 
-pub mod card_description;
-
 mod lane_ir;
 #[cfg(test)]
 mod tests;
@@ -16,6 +14,7 @@ use crate::{
     bytecode::{encode_str, write_to_vec},
     collections::{handle_table::Handle, hash_map::CaoHashMap},
     compiled_program::{CaoCompiledProgram, Label},
+    prelude::Trace,
     Instruction, VariableId,
 };
 use std::convert::TryFrom;
@@ -76,7 +75,7 @@ pub fn compile(
     let options = compile_options.into().unwrap_or_default();
     let compilation_unit = compilation_unit
         .into_ir_stream(options.recursion_limit)
-        .map_err(|err| CompilationError::with_loc(err, CardIndex::default()))?;
+        .map_err(|err| CompilationError::with_loc(err, Trace::default()))?;
 
     let mut compiler = Compiler::new();
     compiler.compile(&compilation_unit, options)
@@ -103,6 +102,13 @@ impl<'a> Compiler<'a> {
         }
     }
 
+    fn trace(&self) -> Trace {
+        Trace {
+            namespace: self.current_namespace.to_owned().into_owned(),
+            index: self.current_index.clone(),
+        }
+    }
+
     pub fn compile(
         &mut self,
         compilation_unit: LaneSlice<'a>,
@@ -112,7 +118,7 @@ impl<'a> Compiler<'a> {
         if compilation_unit.is_empty() {
             return Err(CompilationError::with_loc(
                 CompilationErrorPayload::EmptyProgram,
-                self.current_index.clone(),
+                self.trace(),
             ));
         }
         self.program = CaoCompiledProgram::default();
@@ -125,7 +131,7 @@ impl<'a> Compiler<'a> {
     }
 
     fn error(&self, pl: CompilationErrorPayload) -> CompilationError {
-        CompilationError::with_loc(pl, self.current_index.clone())
+        CompilationError::with_loc(pl, self.trace())
     }
 
     /// build the jump table and consume the lane names
@@ -189,17 +195,14 @@ impl<'a> Compiler<'a> {
         }
 
         for (il, lane) in lanes {
-            self.current_index = CardIndex {
-                lane: il,
-                ..Default::default()
-            };
-            let nodeid_hash = self.current_index.as_handle();
+            self.current_index = CardIndex::lane(il);
+            let nodeid_handle = self.current_index.as_handle();
             let handle = u32::try_from(self.program.bytecode.len())
                 .expect("bytecode length to fit into 32 bits");
             self.program
                 .labels
                 .0
-                .insert(nodeid_hash, Label::new(handle))
+                .insert(nodeid_handle, Label::new(handle))
                 .unwrap();
 
             self.scope_begin();
@@ -394,7 +397,6 @@ impl<'a> Compiler<'a> {
     fn push_str(&mut self, data: &str) {
         let handle = self.program.data.len() as u32;
         write_to_vec(handle, &mut self.program.bytecode);
-
         encode_str(data, &mut self.program.data);
     }
 
@@ -435,7 +437,7 @@ impl<'a> Compiler<'a> {
                     i,
                     k,
                     v,
-                    variable,
+                    iterable: variable,
                     body,
                 } = fe.as_ref();
                 self.current_index.push_subindex(0);
@@ -543,12 +545,23 @@ impl<'a> Compiler<'a> {
                 self.read_var_card(variable)?;
             }
             Card::SetVar(var) => {
-                let index = match self.resolve_var(var.as_str())? {
-                    Some(i) => i as u32,
-                    None => self.add_local(&var)?,
-                };
+                let var = var.as_str();
+                match var.rsplit_once('.') {
+                    Some((read_props, set_prop)) => {
+                        self.read_var_card(read_props)?;
+                        self.push_instruction(Instruction::StringLiteral);
+                        self.push_str(set_prop);
+                        self.push_instruction(Instruction::SetProperty);
+                    }
+                    None => {
+                        let index = match self.resolve_var(var)? {
+                            Some(i) => i as u32,
+                            None => self.add_local(&var)?,
+                        };
 
-                self.write_local_var(index);
+                        self.write_local_var(index);
+                    }
+                }
             }
             Card::SetGlobalVar(variable) => {
                 let next_var = &mut self.next_var;
@@ -606,7 +619,13 @@ impl<'a> Compiler<'a> {
                 self.encode_if_then(Instruction::GotoIfFalse, |c| c.process_card(jmp))?;
                 self.current_index.pop_subindex();
             }
-            Card::Jump(jmp) => self.encode_jump(jmp)?,
+            Card::Jump(jmp) => {
+                self.current_index.push_subindex(0);
+                self.push_instruction(Instruction::FunctionPointer);
+                self.encode_jump(jmp)?;
+                self.current_index.pop_subindex();
+                self.push_instruction(Instruction::CallLane);
+            }
             Card::StringLiteral(c) => self.push_str(c.as_str()),
             Card::CallNative(c) => {
                 let name = &c.0;
@@ -619,6 +638,9 @@ impl<'a> Compiler<'a> {
             Card::ScalarFloat(s) => {
                 write_to_vec(*s, &mut self.program.bytecode);
             }
+            Card::Function(fname) => {
+                self.encode_jump(fname)?;
+            }
             Card::ScalarNil
             | Card::Return
             | Card::And
@@ -641,12 +663,21 @@ impl<'a> Compiler<'a> {
             | Card::Len
             | Card::GetProperty
             | Card::SetProperty
-            | Card::ClearStack => { /* These cards translate to a single instruction */ }
+            | Card::ClearStack
+            | Card::DynamicJump => { /* These cards translate to a single instruction */ }
         }
         Ok(())
     }
 
-    fn read_var_card(&mut self, variable: &str) -> CompilationResult<()> {
+    fn read_var_card(&mut self, mut variable: &str) -> CompilationResult<()> {
+        // sub properties on a table
+        let props = match variable.split_once('.') {
+            Some((v, props)) => {
+                variable = v;
+                props
+            }
+            None => "",
+        };
         let scope = self.resolve_var(variable)?;
         match scope {
             Some(index) => {
@@ -657,7 +688,7 @@ impl<'a> Compiler<'a> {
                 // global
                 let next_var = &mut self.next_var;
                 let varhash = Handle::from_bytes(variable.as_bytes());
-                let id = self
+                let id = *self
                     .program
                     .variables
                     .ids
@@ -667,7 +698,6 @@ impl<'a> Compiler<'a> {
                         *next_var = VariableId(id.0 + 1);
                         id
                     });
-                let id = *id;
                 self.program
                     .variables
                     .names
@@ -677,6 +707,12 @@ impl<'a> Compiler<'a> {
                 write_to_vec(id, &mut self.program.bytecode);
             }
         }
+        // handle props
+        for prop in props.split(".").filter(|p| !p.is_empty()) {
+            self.push_instruction(Instruction::StringLiteral);
+            self.push_str(prop);
+            self.push_instruction(Instruction::GetProperty);
+        }
         Ok(())
     }
 
@@ -700,10 +736,7 @@ impl<'a> Compiler<'a> {
     fn push_instruction(&mut self, instruction: Instruction) {
         self.program
             .trace
-            .insert(
-                self.program.bytecode.len() as u32,
-                self.current_index.clone(),
-            )
+            .insert(self.program.bytecode.len() as u32, self.trace())
             .unwrap();
         self.program.bytecode.push(instruction as u8);
     }
