@@ -32,6 +32,8 @@ pub enum Card {
     CreateTable,
     Abort,
     Len,
+    /// Pop the table, key, value from the stack
+    /// Insert value at key into the table
     SetProperty,
     GetProperty,
     ScalarInt(i64),
@@ -44,9 +46,14 @@ pub enum Card {
     IfElse(Box<[Card; 2]>),
     /// Lane name
     Jump(String),
+    /// Lane name
+    ///
+    /// Creates a pointer to the given cao-lang function
+    Function(String),
     SetGlobalVar(VarName),
     SetVar(VarName),
     ReadVar(VarName),
+    /// Pops the stack for an Integer N and repeats the `body` N times
     Repeat {
         /// Loop variable is written into this variable
         i: Option<VarName>,
@@ -57,6 +64,8 @@ pub enum Card {
     ForEach(Box<ForEach>),
     /// Single card that decomposes into multiple cards
     CompositeCard(Box<CompositeCard>),
+    /// Jump to the function that's on the top of the stack
+    DynamicJump,
 }
 
 #[derive(Debug, Clone)]
@@ -68,8 +77,8 @@ pub struct ForEach {
     pub k: Option<VarName>,
     /// The value is written into this variable
     pub v: Option<VarName>,
-    /// Variable that is iterated on
-    pub variable: Box<Card>,
+    /// Table that is iterated on
+    pub iterable: Box<Card>,
     pub body: Box<Card>,
 }
 
@@ -120,6 +129,8 @@ impl Card {
             Card::SetProperty => "SetProperty",
             Card::ForEach { .. } => "ForEach",
             Card::CompositeCard(c) => c.ty.as_str(),
+            Card::Function(_) => "Function",
+            Card::DynamicJump => "Dynamic Jump",
         }
     }
 
@@ -156,16 +167,18 @@ impl Card {
             Card::Pop => Some(Instruction::Pop),
             Card::ScalarInt(_) => Some(Instruction::ScalarInt),
             Card::ScalarFloat(_) => Some(Instruction::ScalarFloat),
+            Card::Function(_) => Some(Instruction::FunctionPointer),
             Card::CallNative(_) => Some(Instruction::Call),
             Card::IfTrue(_) => None,
             Card::IfFalse(_) => None,
-            Card::Jump(_) => Some(Instruction::CallLane),
+            Card::Jump(_) => None,
             Card::StringLiteral(_) => Some(Instruction::StringLiteral),
             Card::SetGlobalVar(_) => Some(Instruction::SetGlobalVar),
             Card::ClearStack => Some(Instruction::ClearStack),
             Card::ScalarNil => Some(Instruction::ScalarNil),
             Card::Return => Some(Instruction::Return),
             Card::Len => Some(Instruction::Len),
+            Card::DynamicJump => Some(Instruction::CallLane),
         }
     }
 
@@ -210,6 +223,7 @@ impl Card {
             | Instruction::GotoIfTrue
             | Instruction::GotoIfFalse
             | Instruction::ForEach
+            | Instruction::FunctionPointer
             | Instruction::BeginForEach => {}
         };
     }
@@ -265,6 +279,10 @@ impl Card {
         Self::Jump(s.into())
     }
 
+    pub fn function_value(s: impl Into<String>) -> Self {
+        Self::Function(s.into())
+    }
+
     pub fn get_child_mut(&mut self, i: usize) -> Option<&mut Card> {
         let res;
         match self {
@@ -280,7 +298,7 @@ impl Card {
                     i: _,
                     k: _,
                     v: _,
-                    variable: a,
+                    iterable: a,
                     body: b,
                 } = fe.as_mut();
                 if i > 1 {}
@@ -311,7 +329,7 @@ impl Card {
                     i: _,
                     k: _,
                     v: _,
-                    variable: a,
+                    iterable: a,
                     body: b,
                 } = fe.as_ref();
                 if i > 1 {}
@@ -348,7 +366,7 @@ impl Card {
                     i: _,
                     k: _,
                     v: _,
-                    variable: a,
+                    iterable: a,
                     body: b,
                 } = fe.as_mut();
                 if i > 1 {}
@@ -393,7 +411,7 @@ impl Card {
                     i: _,
                     k: _,
                     v: _,
-                    variable: a,
+                    iterable: a,
                     body: b,
                 } = fe.as_mut();
                 if i > 1 {}
@@ -431,7 +449,7 @@ impl Card {
                     i: _,
                     k: _,
                     v: _,
-                    variable: a,
+                    iterable: a,
                     body: b,
                 } = fe.as_mut();
                 match i {
