@@ -7,9 +7,9 @@ fn composite_card_test() {
         Lane::default().with_card(Card::CompositeCard(Box::new(CompositeCard {
             ty: "triplepog".to_string(),
             cards: vec![
-                Card::StringLiteral(StringNode("poggers".to_owned())),
-                Card::StringLiteral(StringNode("poggers".to_owned())),
-                Card::StringLiteral(StringNode("poggers".to_owned())),
+                Card::StringLiteral("poggers".to_owned()),
+                Card::StringLiteral("poggers".to_owned()),
+                Card::StringLiteral("poggers".to_owned()),
             ],
         }))),
     )];
@@ -30,7 +30,7 @@ fn can_binary_de_serialize_output() {
         lanes: [(
             "main".into(),
             Lane::default().with_cards(vec![
-                Card::SetGlobalVar(VarNode::from_str_unchecked("asdsdad")),
+                Card::set_global_var("asdsdad"),
                 Card::Pass,
                 Card::Pass,
             ]),
@@ -52,7 +52,7 @@ fn empty_varname_is_error() {
         submodules: Default::default(),
         lanes: [(
             "main".into(),
-            Lane::default().with_cards(vec![Card::SetGlobalVar(VarNode::from_str_unchecked(""))]),
+            Lane::default().with_cards(vec![Card::set_global_var("")]),
         )]
         .into(),
     };
@@ -80,7 +80,7 @@ fn can_call_nested_function_test() {
         submodules,
         lanes: vec![(
             "main".into(),
-            Lane::default().with_cards(vec![Card::Jump(LaneNode("coggers.pooh".to_string()))]),
+            Lane::default().with_cards(vec![Card::Jump("coggers.pooh".to_string())]),
         )],
     };
 
