@@ -8,7 +8,6 @@ pub use crate::traits::*;
 pub use crate::value::*;
 pub use crate::{
     collections::handle_table::Handle,
-    subprogram_description,
     vm::{runtime::FieldTable, Vm},
-    InputString, StrPointer, SubProgramDescription, SubProgramType,
+    InputString, StrPointer,
 };
