@@ -67,6 +67,10 @@ impl<'a, Aux> Vm<'a, Aux> {
             }
             OwnedValue::Integer(x) => Value::Integer(*x),
             OwnedValue::Real(x) => Value::Real(*x),
+            OwnedValue::Function { hash, arity } => Value::Function {
+                hash: *hash,
+                arity: *arity,
+            },
         };
         Ok(res)
     }
@@ -189,7 +193,6 @@ impl<'a, Aux> Vm<'a, Aux> {
 
     /// This mostly assumes that program is valid, produced by the compiler.
     /// As such running non-compiler emitted programs is very un-safe
-    #[inline(never)]
     pub fn run(&mut self, program: &CaoCompiledProgram) -> ExecutionResult<()> {
         self.runtime_data
             .call_stack
@@ -199,7 +202,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                 stack_offset: 0,
             })
             .map_err(|_| ExecutionErrorPayload::CallStackOverflow)
-            .map_err(|pl| ExecutionError::new(pl, Vec::new()))?;
+            .map_err(|pl| ExecutionError::new(pl, Default::default()))?;
 
         let len = program.bytecode.len();
         let mut remaining_iters = self.max_instr;
@@ -259,7 +262,7 @@ impl<'a, Aux> Vm<'a, Aux> {
                     })?;
                 }
                 Instruction::SetProperty => {
-                    let [value, key, instance] = self.runtime_data.value_stack.pop_n::<3>();
+                    let [key, instance, value] = self.runtime_data.value_stack.pop_n::<3>();
                     let table = self.get_table_mut(instance).map_err(|err| {
                         payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
                     })?;
@@ -382,6 +385,22 @@ impl<'a, Aux> Vm<'a, Aux> {
                         payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
                     })?;
                 }
+                Instruction::FunctionPointer => {
+                    self.runtime_data
+                        .value_stack
+                        .push(Value::Function {
+                            hash: unsafe {
+                                instr_execution::decode_value(&program.bytecode, &mut instr_ptr)
+                            },
+                            arity: unsafe {
+                                instr_execution::decode_value(&program.bytecode, &mut instr_ptr)
+                            },
+                        })
+                        .map_err(|_| ExecutionErrorPayload::Stackoverflow)
+                        .map_err(|err| {
+                            payload_to_error(err, instr_ptr, &self.runtime_data.call_stack)
+                        })?;
+                }
                 Instruction::ScalarInt => {
                     self.runtime_data
                         .value_stack
